/////////////////////////////////////////////////////////////////////////////
//                                                                            
// WWW                 WW                    WW                               
//  WW                 WW                    WW                               
//  WW                 WW                    WW                               
//  WW    WWW    WWW   WW  WW  WW WW  WWWW   WWWW                             
//  WW   WW WW  WW WW  WW WW   WW WW  WW WW  WW WW                            
//  WW   WW WW  WW WW  WWWW    WW WW  WW WW  WW WW                            
//  WW   WW WW  WW WW  WW WW   WW WW  WW WW  WW WW                            
// WWWW   WWW    WWW   WW  WW   WWWW  WWWW   WW WW                            
//                                    WW                                      
//                                    WW                                      
//
// Implements lookup table for sin and cos.
// Kevin Beason <beason@cs.fsu.edu>
// 10/24/02

#ifndef LOOKUP_H
#define LOOKUP_H 1

#include<math.h>
#include<stdlib.h>
#include<stdio.h>

class LookupTable {
private:
  static const int DIM=1024;
  static const int DIM4=DIM/4;
  static const double pi2 = 2.0*M_PI;
  static const double npi2 = -2.0*M_PI;
  static const int DIMB = 8192;
  static const double BESSEL_MAX = 24.0;
  static const double BESSEL_MAX2 = 2.0*BESSEL_MAX;
  double SinT[DIM];
  double BesT[DIMB];
  
public:
  void calcTable();
  
  inline double lsin(double x){
    return (SinT[int(x/pi2 * DIM)%DIM]);
  }
  
  inline double lnsin(double x){
    return (-SinT[int(x/npi2 * DIM)%DIM]);
  }

  inline double lcos(double x){
    return (SinT[(int(x/pi2 * DIM)+DIM4)%DIM]);
  }

  inline double ljn(double x){
    return (BesT[int( (x+BESSEL_MAX)/BESSEL_MAX2 * DIMB)]);
  }
  
};

#endif /* lookup.h */
