/////////////////////////////////////////////////////////////////////////////
//                                                                           
// Includes


#include <Inventor/nodes/SoComplexity.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoShapeHints.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoCube.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoTexture2.h>
#include <Inventor/nodes/SoTexture2Transform.h>
#include <Inventor/nodes/SoPointLight.h>

#include "grid.h"


/////////////////////////////////////////////////////////////////////////////
//                                                                            
// Global Variables

extern  GridStruct       Grid;



/////////////////////////////////////////////////////////////////////////////
//                                                                            
//  WWWWW                                                                     
// WW  WW                         WW                                          
// WW                             WW                                          
// WW      WW WW   WWW    WWWW   WWWW   WWW                                   
// WW      WWW    WW WW      WW   WW   WW WW                                  
// WW      WW     WWWWW   WWWWW   WW   WWWWW                                  
// WW  WW  WW     WW     WW  WW   WW   WW                                     
//  WWWW   WW      WWWW   WWWWW   WW    WWWW                                  
//                                                                            
//                                                                            
//  WWWW                                                                      
// WW  WW                                                                     
// WWW                                                                        
//  WWWW    WWW    WWW   WWWW    WWW                                          
//    WWW  WW WW  WW WW  WW WW  WW WW                                         
//     WW  WW     WWWWW  WW WW  WWWWW                                         
// WW  WW  WW WW  WW     WW WW  WW                                            
//  WWWW    WWW    WWWW  WW WW   WWWW                                         
//                                                                            
//                                                                            
// Create a scene graph with draggers that control waves
// in a 2D mesh.

SoSeparator *
createScene()
  {
  //
  //                              root
  //                                |
  //    -----------------------------
  //    |    |   |        |     |
  // view  com  two      mesh  quad
  //xform  plex side     co    mesh
  //       ity           ord  
  //                   

  SoSeparator *root = new SoSeparator();

  // view orientation
  // Rotate the scene so that the z-axis is nearly vertical,
  // toward the y-direction. Rotate about the x-axis.
  SoTransform *viewTransform = new SoTransform();
    viewTransform->rotation.setValue(SbRotation
      (SbVec3f (1.0, 0.0, 0.0),  -M_PI*0.30) );
    viewTransform->rotation.setValue(
      viewTransform->rotation.getValue()  *
      SbRotation(SbVec3f (0.0, 1.0, 0.0),   M_PI*0.10 ) 
      ); 
    //root->addChild(viewTransform);


  // complexity
  SoComplexity *complexity = new SoComplexity();
    complexity->textureQuality = 1.0;
    complexity->value = 0.8;
  root->addChild(complexity);

  // light
  SoPointLight *roomLight = new SoPointLight();
  roomLight->location.setValue(SbVec3f(0.0, 0.0, 5.0));
  roomLight->intensity.setValue(1.0);
  roomLight->color.setValue(SbColor(0.8, 0.7, 0.7));
  root->addChild( roomLight );


  // INVENTOR HACK: set up the hints to produce two-sided lighting
  SoShapeHints *twoSided = new SoShapeHints();
    twoSided->shapeType = SoShapeHints::UNKNOWN_SHAPE_TYPE;
    twoSided->vertexOrdering = SoShapeHints::COUNTERCLOCKWISE;
  root->addChild(twoSided);


  // Cube Background stuff
  double cubeSize = 10.0;
  SoSeparator *cubeSep = new SoSeparator();
  SoShapeHints *oneSided = new SoShapeHints();
  oneSided->shapeType = SoShapeHints::SOLID;
  oneSided->vertexOrdering = SoShapeHints::COUNTERCLOCKWISE;
  cubeSep->addChild(oneSided);
  SoTexture2Transform *tileTransform = new SoTexture2Transform();
  tileTransform->scaleFactor.setValue(SbVec2f(10.0, 10.0));
  cubeSep->addChild(tileTransform);
  SoTexture2 *tileTexture = new SoTexture2();
  tileTexture->filename.setValue("tile.rgb");
  cubeSep->addChild(tileTexture);
  SoCube *myCube = new SoCube();
  myCube->width.setValue(-cubeSize);
  myCube->depth.setValue(-cubeSize);
  myCube->height.setValue(-cubeSize);
  cubeSep->addChild(myCube);
  root->addChild(cubeSep);

  // invisible cube
  SoSeparator *invisSep = new SoSeparator();
  SoMaterial *invisMat = new SoMaterial();
  invisMat->transparency.setValue(1.0);
  invisSep->addChild(invisMat);
  SoCube *invisCube = new SoCube();
  invisCube->width.setValue(cubeSize);
  invisCube->height.setValue(cubeSize);
  invisCube->depth.setValue(cubeSize);
  invisSep->addChild(invisCube);
  root->addChild(invisSep);
  

  // surface texture
  SoTexture2 *surfTexture = new SoTexture2();
  surfTexture->filename.setValue("water.rgb");
  root->addChild(surfTexture);

  // surface material
  SoMaterial *surfMat = new SoMaterial();
  surfMat->transparency.setValue(0.0);
  root->addChild(surfMat);

  // mesh coordinates
  SoCoordinate3 *coord = new SoCoordinate3();
    coord->point.setNum(Grid.sDim * Grid.tDim);
  root->addChild(coord);
  Grid.coord = coord;


  // quadMesh
  SoQuadMesh *quadMesh = new SoQuadMesh;
    quadMesh->verticesPerRow    = Grid.sDim;
    quadMesh->verticesPerColumn = Grid.tDim;
  root->addChild(quadMesh);
  Grid.mesh = quadMesh;


  return root;
  }

